/*
 * Common system pager functionality.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <set>

#include <exec/memory_area.h>



/* Collection definitions. */

typedef std::set<l4_cap_idx_t> Capabilities;



/* A simple system pager also acting as a region mapper. */

class ExecPager
{
protected:

  /* Top-level area containing other areas and regions. */

  AvailableMemoryArea _area;

  /* Dataspace capabilities associated with regions. */

  Capabilities _dataspaces;

  /* Region manager/mapper functionality. */

  virtual long find(address_t *start, address_t *size, map_flags_t flags,
                    unsigned char align, MemoryArea **area);

public:
  explicit ExecPager(address_t start = 0, address_t end = 0);

  virtual ~ExecPager();

  /* Region management methods. */

  virtual void add(MemoryArea &area);

  virtual void remove(MemoryArea &area);

  /* Notification methods. */

  virtual long exception(l4_exc_regs_t regs,
                         l4_snd_fpage_t *region) = 0;

  virtual long page_fault(l4_umword_t pfa, l4_umword_t pc,
                          l4_snd_fpage_t *region) = 0;
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
